/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "diagramarea.h"
#include "diagramwindow.h"
#include "diagrampopupmenu.h"
#include "dialogs.h"
#include "diagram_graphic.h"
#include <gtk--/style.h>
#include <gdk--/colormap.h>
#include <gdk--/color.h>
#include <gdk--/gc.h>
#include <algorithm>
#include <stdio.h>

#define GTKMM_FULL_ANGLE 23040

void
DiagramArea::coords_diagram_to_area(int &x, int &y, int min_x, int min_y, 
				    int w, int h)
{
  x = x - min_x + ((width() - w) >> 1);
  y = h - (y - min_y) + ((height() - h) >> 1);
}

void
DiagramArea::coords_area_to_diagram(int &x, int &y, int min_x, int min_y,
				    int w, int h)
{
  x = x - ((width() - w) >> 1) + min_x;
  y = h - (y - ((height() - h) >> 1) - min_y);
}

void
DiagramArea::drawVertex(gint x, gint y, string label, bool selected,
                        bool marked, bool hidden_label)
{
  int point_size_2;
  Gdk_Color col = color;
  
  point_size_2 = diagram_vertex_size / 2;
  // color
  if (selected) col.set_rgb(col.get_red() + selected_color.get_red(),
			    col.get_green() + selected_color.get_green(),
			    col.get_blue() + selected_color.get_blue());
  if (marked) col.set_rgb(col.get_red() + marked_color.get_red(),
			  col.get_green() + marked_color.get_green(),
			  col.get_blue() + marked_color.get_blue());
  get_default_colormap().alloc(col);
  gc.set_foreground(col);
  // draw vertex
  pixmap.
    draw_arc(gc, TRUE, x - point_size_2, y - point_size_2, 
  	     diagram_vertex_size, diagram_vertex_size, 0, GTKMM_FULL_ANGLE);
  // draw label
  // TODO: will be an option in preferences
  if (!hidden_label) {
    pixmap.
      draw_rectangle(gc, FALSE, x + diagram_vertex_size - 3,
		     y - text_font.string_height(label) - 2, 
		     text_font.string_width(label) + 4, 
		     text_font.string_height(label) + 3);
    gc.set_foreground(back_color);
    pixmap.
      draw_rectangle(gc, TRUE,
		     x + diagram_vertex_size - 2,
		     y - text_font.string_height(label) - 1, 
		     text_font.string_width(label) + 3, 
		     text_font.string_height(label) + 2);
    gc.set_foreground(col);
    pixmap.
      draw_string(text_font, gc, x + diagram_vertex_size, y, label);
  }
}

void
DiagramArea::drawDiagram()
{
  set<Vertex> verteces;
  set<Vertex>::iterator iter_s1, iter_s2;
  Element *elem = NULL;
  set<Element>::iterator iter_se;
  int vx1, vy1, vx2, vy2;
  Coord_t x1, y1, x2, y2, w, h;
  Vertex vert(NoElement);

  orderedset->getDiagram()->getBox(x1, y1, x2, y2);
  w = x2 - x1; h = y2 - y1;
  // draw background
  gc.set_foreground(back_color);
  pixmap.draw_rectangle(gc, TRUE, 0, 0, width(), height());
  orderedset->getDiagram()->getVerteces(verteces);
  gc.set_foreground(color);
  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vx1 = (*iter_s1).x; vy1 = (*iter_s1).y;
    coords_diagram_to_area(vx1, vy1, x1, y1, w, h);
    for (iter_s2 = verteces.begin(); iter_s2 != verteces.end(); iter_s2++) {
      if (*iter_s1 == *iter_s2) continue;
      if (orderedset->getDiagram()->isEdge(*iter_s1, *iter_s2)) {
	vx2 = (*iter_s2).x; vy2 = (*iter_s2).y;
	coords_diagram_to_area(vx2, vy2, x1, y1, w, h);
	// draw edge
	pixmap.draw_line(gc, vx1, vy1, vx2, vy2);
      }
    }
  }
  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vert = *iter_s1;
    vx1 = vert.x; vy1 = vert.y;
    coords_diagram_to_area(vx1, vy1, x1, y1, w, h);
    elem = orderedset->getDiagram()->getElement(vert);
    if (elem) {
      iter_se = selected_elements.find(*elem);
      if (iter_se != selected_elements.end())
	vert.properties |= Vertex::SELECTED;
      else vert.properties &= (unsigned long)~Vertex::SELECTED;
      iter_se = marked_elements.find(*elem);
      if (iter_se != marked_elements.end())
	vert.properties |= Vertex::MARKED;
      else vert.properties &= (unsigned long)~Vertex::MARKED;
      orderedset->getDiagram()->updateVertex(vert);
      drawVertex(vx1, vy1, elem->name, vert.properties & Vertex::SELECTED,
		 vert.properties & Vertex::MARKED, 
		 vert.properties & Vertex::HIDDEN_LABEL);
    }
  }
  orderedset->setUpdated();
}

void
DiagramArea::flush(int x, int y, int w, int h)
{
  //Gdk_Pixmap null_pixmap;
  Gdk_Window window = get_window();
  
  gc.set_foreground(back_color);
  back_pixmap.draw_rectangle(gc, TRUE, x, y, w, h);
  back_pixmap.draw_pixmap(gc, pixmap, x, y, x, y, w, h);
  window.set_back_pixmap(back_pixmap, false);
  window.clear_area(x, y, w, h);
  //window.set_back_pixmap(null_pixmap, false);
}

gint
DiagramArea::configure_event_impl(GdkEventConfigure* ev)
{
  Gtk::DrawingArea::configure_event_impl(ev);
  pixmap.create(get_window(), ev->width, ev->height);
  back_pixmap.create(get_window(), ev->width, ev->height);
  if (!gc) gc.create(pixmap);
  drawDiagram();
  return FALSE;
}

gint
DiagramArea::expose_event_impl(GdkEventExpose *ev)
{
  flush(ev->area.x, ev->area.y, ev->area.width, ev->area.height);
  return Gtk::DrawingArea::expose_event_impl(ev);
}

gint
DiagramArea::button_press_event_impl(GdkEventButton *ev)
{
  int x, y;
  Vertex *vert_p;
  Vertex vert(NoElement);
  set<Element>::iterator iter_s;
  NewElementDialog *new_dialog;
  string name;
  Element *elem = NULL;
  Coord_t x1, y1, x2, y2, w, h;
  list<set<Element> >::iterator iter_l;

  x = (int)ev->x; y = (int)ev->y;
  orderedset->getDiagram()->getBox(x1, y1, x2, y2);
  w = x2 - x1; h = y2 - y1;
  coords_area_to_diagram(x, y, x1, y1, w, h);
  if (ev->button == 1) {
    vert_p = orderedset->getDiagram()->
      getVertexInBox(x, y, diagram_vertex_size / 2);
    if (vert_p) elem = orderedset->getDiagram()->getElement(*vert_p);
    if (popupmenu->getMode() == DiagramPopupMenu::MODE_SELECT) {
      drag_vertex = vert_p;
      drag_old_diagram_coord_x = x; drag_old_diagram_coord_y = y;
      if (elem) {
	if (ev->state & GDK_SHIFT_MASK) {
	  iter_s = selected_elements.find(*elem);
	  if (iter_s == selected_elements.end()) 
	    selected_elements.insert(*elem);
	  else selected_elements.erase(iter_s);
	}
	mode_elements.clear();
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_NEW) {
      new_dialog = new NewElementDialog;
      vert = Vertex(NoElement, NoElement, x, y);
      new_dialog->exec();
      name = new_dialog->getElementName();
      delete new_dialog;
      if (!name.empty()) {
	orderedset->save_UndoRedo();
	orderedset->newElement(name, &vert);
      }
      mode_elements.clear();
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_REMOVE) {
      if (elem) {
	orderedset->save_UndoRedo();
	orderedset->removeElement(*elem);
	selected_elements.erase(*elem);
	for (iter_l = path_elems.begin(); iter_l != path_elems.end();
	     iter_l++)
	  (*iter_l).erase(*elem);
	selectpath_dialog->setPaths(path_elems);
	mode_elements.clear();
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_COVER) {
      if (elem) {
	mode_elements.push_back(*elem);
	if (mode_elements.size() == 2) {
	  orderedset->save_UndoRedo();
	  orderedset->orderElements(mode_elements.front(),
				    mode_elements.back());
	  mode_elements.clear();
	}
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_REMOVE_COVER) {
      if (elem) {
	mode_elements.push_back(*elem);
	if (mode_elements.size() == 2) {
	  orderedset->save_UndoRedo();
	  orderedset->removeOrder(mode_elements.front(), 
				  mode_elements.back());
	  mode_elements.clear();
	}
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_MARK_UPPER) {
      if (elem) {
	marked_elements.clear();
	orderedset->upperElements(marked_elements, *elem);
	mode_elements.clear();
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_MARK_LOWER) {
      if (elem) {
	marked_elements.clear();
	orderedset->lowerElements(marked_elements, *elem);
	mode_elements.clear();
      }
    }
    else if (popupmenu->getMode() == DiagramPopupMenu::MODE_MARK_PATHS) {
      if (elem) {
	mode_elements.push_back(*elem);
	if (mode_elements.size() == 2) {
	  path_elems.clear();
	  orderedset->getPaths(path_elems, mode_elements.front(), 
			       mode_elements.back());
	  mode_elements.clear();
	  marked_elements.clear();
	  selectpath_dialog->setPaths(path_elems);
	  if (path_elems.size() > 1) selectpath_dialog->exec();
	  else if (!path_elems.empty()) {
	    marked_elements = path_elems.front();
	  }
	}
      }
    }
    drawDiagram();
    flush(0, 0, width(), height());
    window->setTitle();
  }
  else if (ev->button == 3) {
    mode_elements.clear();
    popupmenu->popup(x, y, ev->button, ev->time);
  }
  return FALSE;
}

gint 
DiagramArea::motion_notify_event_impl(GdkEventMotion *ev)
{
  int x, y;
  Coord_t dx, dy;
  Vertex *vert;
  set<Element>::iterator iter_s;
  Coord_t x1, y1, x2, y2, w, h;

  x = (int)ev->x; y = (int)ev->y;
  if ((ev->state & GDK_BUTTON1_MASK) &&
      (x > diagram_border_x) && (x < (width() - diagram_border_x)) &&
      (y > diagram_border_y) && (y < (height() - diagram_border_y))) {
    orderedset->getDiagram()->getBox(x1, y1, x2, y2);
    w = x2 - x1; h = y2 - y1;
    coords_area_to_diagram(x, y, x1, y1, w, h);
    dx = x - drag_old_diagram_coord_x; dy = y - drag_old_diagram_coord_y;
    if ((!(ev->state & GDK_SHIFT_MASK)) && (drag_vertex != NULL)) {
      drag_vertex->x += dx; drag_vertex->y += dy;
      drag_vertex = orderedset->getDiagram()->updateVertex(*drag_vertex);
    }
    if ((ev->state & GDK_SHIFT_MASK) && (!(selected_elements.empty())))
      for (iter_s = selected_elements.begin(); 
	   iter_s != selected_elements.end(); iter_s++) {
	vert = orderedset->getDiagram()->getVertex(*iter_s);
	vert->x += dx; vert->y += dy;
	orderedset->getDiagram()->updateVertex(*vert);
      }
    drag_old_diagram_coord_x = x; drag_old_diagram_coord_y = y;
    drawDiagram();
    flush(0, 0, width(), height());
    window->setTitle();
  }
  return FALSE;
}

DiagramArea::DiagramArea(DiagramWindow *win, DiagramPopupMenu *pmenu, 
			 OrderedSets *oss, OrderedSet *os, int min_width,
			 int min_height)
  : diagram_vertex_size(10),
    diagram_border_x(20), diagram_border_y(20),
    color("black"), back_color("white"), selected_color("green"),
    marked_color("blue"), text_font("fixed")
{
  Coord_t x1, y1, x2, y2, w, h;
  Gdk_Colormap colormap = get_default_colormap();

  orderedsets = oss;
  orderedset = os;
  window = win;
  popupmenu = pmenu;
  drag_vertex = NULL;
  min_diagram_width = min_width; min_diagram_height = min_height;
  set_events(GDK_EXPOSURE_MASK | 
	     GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK |
	     GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK | 
	     GDK_KEY_RELEASE_MASK);
  colormap.alloc(color); colormap.alloc(back_color);
  colormap.alloc(selected_color); colormap.alloc(marked_color);
  orderedset->getDiagram()->getBox(x1, y1, x2, y2);
  w = x2 - x1; h = y2 - y1;
  // size
  if (w < min_diagram_width) w = min_diagram_width;
  if (h < min_diagram_height) h = min_diagram_height;
  size(w + 2 * diagram_border_x, h + 2 * diagram_border_y);
  selectpath_dialog = new SelectPathDialog(this);
  window->setTitle();
  show();
}

DiagramArea::~DiagramArea()
{
  delete selectpath_dialog;
}

void
DiagramArea::update()
{
  Coord_t x1, y1, x2, y2, w, h;

  orderedset->getDiagram()->getBox(x1, y1, x2, y2);
  w = x2 - x1; h = y2 - y1;
  // size
  if (w < min_diagram_width) w = min_diagram_width;
  if (h < min_diagram_height) h = min_diagram_height;
  if ((w > width()) || (h > height()))
    size(w + 2 * diagram_border_x, h + 2 * diagram_border_y);
  drawDiagram();
  flush(0, 0, width(), height());
  window->setTitle();
}

void
DiagramArea::removeSelectedElements()
{
  list<set<Element> >::iterator iter_l;
  set<Element>::iterator iter_s;

  orderedset->save_UndoRedo();
  for (iter_s = selected_elements.begin(); iter_s != selected_elements.end();
       iter_s++) {
    orderedset->removeElement(*iter_s);
    for (iter_l = path_elems.begin(); iter_l != path_elems.end();
	 iter_l++)
      (*iter_l).erase(*iter_s);
  }
  selectpath_dialog->setPaths(path_elems);
  selected_elements.clear();
}

void 
DiagramArea::invertSelectedElements()
{
  set<Element> elems, invert;
  
  orderedset->getElements(elems);
  set_difference(elems.begin(), elems.end(), selected_elements.begin(),
		 selected_elements.end(), insert_iterator<set<Element> >
                 (invert, invert.begin()));
  selected_elements = invert;
}
