/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "diagrampopupmenu.h"
#include "appfilemenu.h"
#include "diagramwindow.h"
#include "diagramarea.h"
#include "appwindow.h"
#include "appbuttons.h"
#include "dialogs.h"
#include "saver.h"
#include "diagram_graphic.h"
#include "i18n.h"
#include <stdio.h>

void
DiagramPopupMenu::save()
{
  string filename;

  filename = orderedset->getFilename();
  if (filename == "") saveasfile_dialog->exec();
  else {
    saveasfile_dialog->saveFile(filename);
    window->update();
  }
}

void
DiagramPopupMenu::saveAs()
{
  saveasfile_dialog->exec();
}

void
DiagramPopupMenu::undo()
{
  orderedset->undo();
  window->update();
}

void
DiagramPopupMenu::redo()
{
  orderedset->redo();
  window->update();
}

void
DiagramPopupMenu::cut()
{
  set<Element> elems;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  copy();
  area->removeSelectedElements();
  window->update();
}

void
DiagramPopupMenu::copy()
{
  OrderedSet *copy_os;
  set<Element> elems;
  set<Element>::iterator iter_s1, iter_s2;
  list<Element *> new_elems;
  list<Element *>::iterator iter_l1, iter_l2;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  copy_os = appwindow->getCopyOrderedSet();
  copy_os->clear();
  for (iter_s1 = elems.begin(); iter_s1 != elems.end(); iter_s1++)
    new_elems.
      push_back(copy_os->newElement((*iter_s1).name,
				    orderedset->getDiagram()->
				    getVertex(*iter_s1)));
  for (iter_s1 = elems.begin(), iter_l1 = new_elems.begin();
       iter_s1 != elems.end(); iter_s1++, iter_l1++)
    for (iter_s2 = elems.begin(), iter_l2 = new_elems.begin();
	 iter_s2 != elems.end(); iter_s2++, iter_l2++)
      if (orderedset->less(*iter_s1, *iter_s2, false) && *iter_l1 && *iter_l2)
	copy_os->orderElements(**iter_l1, **iter_l2);
}

void
DiagramPopupMenu::paste()
{
  OrderedSet *copy_os;
  set<Element> elems;
  set<Element>::iterator iter_s1, iter_s2;
  list<Element *> new_elems;
  list<Element *>::iterator iter_l1, iter_l2;

  copy_os = appwindow->getCopyOrderedSet();
  copy_os->getElements(elems);
  if (elems.empty()) return;
  orderedset->save_UndoRedo();
  for (iter_s1 = elems.begin(); iter_s1 != elems.end(); iter_s1++)
    new_elems.
      push_back(orderedset->newElement((*iter_s1).name,
				       copy_os->getDiagram()->
				       getVertex(*iter_s1)));
  for (iter_s1 = elems.begin(), iter_l1 = new_elems.begin();
       iter_s1 != elems.end(); iter_s1++, iter_l1++)
    for (iter_s2 = elems.begin(), iter_l2 = new_elems.begin();
	 iter_s2 != elems.end(); iter_s2++, iter_l2++)
      if (copy_os->less(*iter_s1, *iter_s2, false) && *iter_l1 && *iter_l2)
	orderedset->orderElements(**iter_l1, **iter_l2);
  window->update();
}

void
DiagramPopupMenu::clear()
{
  orderedset->save_UndoRedo();
  orderedset->clear();
  window->update();
}

void
DiagramPopupMenu::selectElements()
{
  appbuttons->selectButton(AppButtons::SELECT);
}

void
DiagramPopupMenu::selectInvert()
{
  area->invertSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectAll()
{
  area->clearSelectedElements();
  area->invertSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectNone()
{
  area->clearSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectMarked()
{
  area->selectMarkedElements();
  window->update();
}

void
DiagramPopupMenu::newDiagramView()
{
  int width, height;
  
  area->getMinDiagramSize(width, height);
  orderedsets->refSet(orderedset);
  SigC::manage(new DiagramWindow(appwindow, orderedsets, orderedset,
                                 width, height));
}

void
DiagramPopupMenu::newElements()
{
  appbuttons->selectButton(AppButtons::NEW);
}

void
DiagramPopupMenu::renameElement()
{
  Vertex *vert_p;
  Element *elem = NULL;
  NewElementDialog *new_dialog;
  string name;
  
  vert_p = orderedset->getDiagram()->
    getVertexInBox(x_menu, y_menu, area->getDiagramVertexSize() / 2);
  if (vert_p) elem = orderedset->getDiagram()->getElement(*vert_p);
  if (elem) {
    new_dialog = new NewElementDialog(true);
    new_dialog->exec();
    name = new_dialog->getElementName();
    delete new_dialog;
    if (!name.empty()) {
      orderedset->save_UndoRedo();
      orderedset->rename(*elem, name);
      window->update();
    }
  }
}

void
DiagramPopupMenu::removeElements()
{
  appbuttons->selectButton(AppButtons::REMOVE);
}

void
DiagramPopupMenu::coverElements()
{
  appbuttons->selectButton(AppButtons::COVER);
}

void
DiagramPopupMenu::removeCover()
{
  appbuttons->selectButton(AppButtons::REMOVE_COVER);
}

void
DiagramPopupMenu::rearrangeDiagram()
{
  rearrangediagram_dialog->exec();
}

void
DiagramPopupMenu::markUpper()
{
  appbuttons->selectButton(AppButtons::MARK_UPPER);
}

void
DiagramPopupMenu::markLower()
{
  appbuttons->selectButton(AppButtons::MARK_LOWER);
}

void
DiagramPopupMenu::markMinimal()
{
  set<Element> s;

  orderedset->minimalElements(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markMaximal()
{
  set<Element> s;

  orderedset->maximalElements(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markLeast()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->leastElement(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markGreatest()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->greatestElement(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markLowerCone()
{
  set<Element> s;

  orderedset->lowerCone(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markUpperCone()
{
  set<Element> s;

  orderedset->upperCone(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markInfimum()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->infimum(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markSupremum()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->supremum(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markPaths()
{
  appbuttons->selectButton(AppButtons::MARK_PATHS);
}

void
DiagramPopupMenu::unmarkAll()
{
  area->clearMarkedElements();
  window->update();
}

void
DiagramPopupMenu::hideVertecesLabels()
{
  set<Element> elems;
  set<Element>::iterator iter_s;
  Vertex *vert = NULL;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  for (iter_s = elems.begin(); iter_s != elems.end(); iter_s++) {
    vert = orderedset->getDiagram()->getVertex(*iter_s);
    if (!vert) continue;
    vert->properties |= Vertex::HIDDEN_LABEL;
    orderedset->getDiagram()->updateVertex(*vert);
    delete vert; vert = NULL;
  }
  window->update();
}

void
DiagramPopupMenu::showVertecesLabels()
{
  set<Element> elems;
  set<Element>::iterator iter_s;
  Vertex *vert = NULL;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  for (iter_s = elems.begin(); iter_s != elems.end(); iter_s++) {
    vert = orderedset->getDiagram()->getVertex(*iter_s);
    if (!vert) continue;
    vert->properties &= (unsigned long)~Vertex::HIDDEN_LABEL;
    orderedset->getDiagram()->updateVertex(*vert);
    delete vert; vert = NULL;
  }
  window->update();
}

DiagramPopupMenu::DiagramPopupMenu(AppWindow *appwin, DiagramWindow *win, 
				   OrderedSets *oss, OrderedSet *os)
{
  AppFileMenu *filemenu;
  Gtk::Menu *editmenu, *selectmenu, *viewmenu, *datamenu, *data_markmenu,
    *diagrammenu;
  Gtk::Menu_Helpers::MenuList::iterator iter_l;
  
  orderedsets = oss;
  orderedset = os;
  appwindow = appwin;
  window = win;
  area = NULL;
  appbuttons = appwin->getButtons();
  x_menu = y_menu = 0;
  // file
  filemenu = SigC::manage(new AppFileMenu(appwin, oss));
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("File"), *filemenu));
  // save
  iter_l = filemenu->items().begin(); iter_l++; iter_l++; iter_l++;
  filemenu->items().insert
    (iter_l, Gtk::Menu_Helpers::MenuElem
     (_("Save"), Gtk::Menu_Helpers::CTL|'S',
      SigC::slot(*this, &DiagramPopupMenu::save)));
  // save as
  filemenu->items().insert
    (iter_l, Gtk::Menu_Helpers::MenuElem
     (_("Save As..."), SigC::slot(*this, &DiagramPopupMenu::saveAs)));
  // close
  iter_l = filemenu->items().end(); iter_l--;
  iter_l = filemenu->items().insert
    (iter_l, Gtk::Menu_Helpers::MenuElem
     (_("Close"), Gtk::Menu_Helpers::CTL|'W',
      SigC::slot(*window, &DiagramWindow::close)));
  // edit
  editmenu = SigC::manage(new Gtk::Menu);
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("Edit"), *editmenu));
  // tearoff
  editmenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // undo
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Undo"), Gtk::Menu_Helpers::CTL|'Z',
      SigC::slot(*this, &DiagramPopupMenu::undo)));
  // redo
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Redo"), Gtk::Menu_Helpers::CTL|'R',
      SigC::slot(*this, &DiagramPopupMenu::redo)));
  // separator
  editmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // cut
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Cut"), Gtk::Menu_Helpers::CTL|'X',
      SigC::slot(*this, &DiagramPopupMenu::cut)));
  // copy
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Copy"), Gtk::Menu_Helpers::CTL|'C',
      SigC::slot(*this, &DiagramPopupMenu::copy)));
  // paste
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Paste"), Gtk::Menu_Helpers::CTL|'V',
      SigC::slot(*this, &DiagramPopupMenu::paste)));
  // separator
  editmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // clear
  editmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Clear"), Gtk::Menu_Helpers::CTL|'K',
      SigC::slot(*this, &DiagramPopupMenu::clear)));
  // select
  selectmenu = SigC::manage(new Gtk::Menu);
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("Select"), *selectmenu));
  // tearoff
  selectmenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // select elements
  selectmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Select elements"), 'S', 
      SigC::slot(*this, &DiagramPopupMenu::selectElements)));
  // separator
  selectmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // invert
  selectmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Invert"), Gtk::Menu_Helpers::CTL|'I',
      SigC::slot(*this, &DiagramPopupMenu::selectInvert)));
  // all
  selectmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("All"), Gtk::Menu_Helpers::CTL|'A',
      SigC::slot(*this, &DiagramPopupMenu::selectAll)));
  // none
  selectmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("None"), Gtk::Menu_Helpers::SHIFT|Gtk::Menu_Helpers::CTL|'A',
      SigC::slot(*this, &DiagramPopupMenu::selectNone)));
  // separator
  selectmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // marked
  selectmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Marked"), Gtk::Menu_Helpers::CTL|'M',
      SigC::slot(*this, &DiagramPopupMenu::selectMarked)));
  // view
  viewmenu = SigC::manage(new Gtk::Menu);
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("View"), *viewmenu));
  // tearoff
  viewmenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // new diagram view
  viewmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("New Diagram View"), 
      SigC::slot(*this, &DiagramPopupMenu::newDiagramView)));
  // diagram
  datamenu = SigC::manage(new Gtk::Menu);
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("Data"), *datamenu));
  // tearoff
  datamenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // insert elements
  datamenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("New elements"), "Insert",
      SigC::slot(*this, &DiagramPopupMenu::newElements)));
  // rename element
  datamenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Rename element..."),
      SigC::slot(*this, &DiagramPopupMenu::renameElement)));
  // remove elements
  datamenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Remove elements"), "Delete",
      SigC::slot(*this, &DiagramPopupMenu::removeElements)));
  // cover two elements
  datamenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Define cover x < y"), 'C',
      SigC::slot(*this, &DiagramPopupMenu::coverElements)));
  // remove cover
  datamenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Remove cover"), Gtk::Menu_Helpers::SHIFT|'C',
      SigC::slot(*this, &DiagramPopupMenu::removeCover)));
  // separator
  datamenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // mark
  data_markmenu = SigC::manage(new Gtk::Menu);
  datamenu->items().push_back(Gtk::Menu_Helpers::MenuElem
			      (_("Mark"), *data_markmenu));
  // tearoff
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::TearoffMenuElem());
  // lower elements
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Lower elements"), Gtk::Menu_Helpers::SHIFT|'L',
      SigC::slot(*this, &DiagramPopupMenu::markLower)));
  // upper elements
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Upper elements"), Gtk::Menu_Helpers::SHIFT|'U',
      SigC::slot(*this, &DiagramPopupMenu::markUpper)));
  // separator
  data_markmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // minimal elements
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Minimal elements"), Gtk::Menu_Helpers::SHIFT|'I',
      SigC::slot(*this, &DiagramPopupMenu::markMinimal)));
  // maximal elements
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Maximal elements"), Gtk::Menu_Helpers::SHIFT|'A',
      SigC::slot(*this, &DiagramPopupMenu::markMaximal)));
  // least element
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Least element"), Gtk::Menu_Helpers::SHIFT|'E',
      SigC::slot(*this, &DiagramPopupMenu::markLeast)));
  // greatest element
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Greatest element"), Gtk::Menu_Helpers::SHIFT|'G',
      SigC::slot(*this, &DiagramPopupMenu::markGreatest)));
  // lower cone
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Lower cone"), Gtk::Menu_Helpers::SHIFT|'W',
      SigC::slot(*this, &DiagramPopupMenu::markLowerCone)));
  // upper cone
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Upper cone"), Gtk::Menu_Helpers::SHIFT|'R',
      SigC::slot(*this, &DiagramPopupMenu::markUpperCone)));
  // infimum element
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Infimum"), Gtk::Menu_Helpers::SHIFT|'F',
      SigC::slot(*this, &DiagramPopupMenu::markInfimum)));
  // supremum element
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Supremum"), Gtk::Menu_Helpers::SHIFT|'S',
      SigC::slot(*this, &DiagramPopupMenu::markSupremum)));
  // separator
  data_markmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // paths
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Paths..."), Gtk::Menu_Helpers::SHIFT|'P',
      SigC::slot(*this, &DiagramPopupMenu::markPaths)));
  // separator
  data_markmenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // unmark all
  data_markmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Unmark all"), Gtk::Menu_Helpers::SHIFT|'M',
      SigC::slot(*this, &DiagramPopupMenu::unmarkAll)));
  // diagram
  diagrammenu = SigC::manage(new Gtk::Menu);
  items().push_back(Gtk::Menu_Helpers::MenuElem(_("Diagram"), *diagrammenu));
  // tearoff
  diagrammenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // rearrange diagram
  diagrammenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Rearrange diagram..."), Gtk::Menu_Helpers::CTL|'D',
      SigC::slot(*this, &DiagramPopupMenu::rearrangeDiagram)));
  // separator
  diagrammenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  // hide verteces labels
  diagrammenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Hide verteces labels"), Gtk::Menu_Helpers::CTL|'H',
      SigC::slot(*this, &DiagramPopupMenu::hideVertecesLabels)));
  // show verteces labels
  diagrammenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("Show verteces labels"), 
      Gtk::Menu_Helpers::SHIFT|Gtk::Menu_Helpers::CTL|'H',
      SigC::slot(*this, &DiagramPopupMenu::showVertecesLabels)));
  accelerate(*win);
  saveasfile_dialog = new SaveAsFileDialog(orderedset, window, appwindow);
  rearrangediagram_dialog = new RearrangeDiagramDialog(orderedset, window,
						       appwindow);
  show();
}

DiagramPopupMenu::~DiagramPopupMenu()
{
  delete saveasfile_dialog;
  delete rearrangediagram_dialog;
}

void
DiagramPopupMenu::popup(int x, int y, guint button, guint32 activate_time)
{
  x_menu = x; y_menu = y;
  // popup
  Menu::popup(button, activate_time);
}

DiagramPopupMenu::modes_t
DiagramPopupMenu::getMode()
{
  switch (appbuttons->getSelectedButton()) {
  case AppButtons::SELECT : return MODE_SELECT;
  case AppButtons::NEW : return MODE_NEW;
  case AppButtons::REMOVE : return MODE_REMOVE;
  case AppButtons::COVER : return MODE_COVER;
  case AppButtons::REMOVE_COVER : return MODE_REMOVE_COVER;
  case AppButtons::MARK_UPPER : return MODE_MARK_UPPER;
  case AppButtons::MARK_LOWER : return MODE_MARK_LOWER;
  case AppButtons::MARK_PATHS : return MODE_MARK_PATHS;
  }
  return MODE_SELECT;
}
