/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "diagramwindow.h"
#include "diagramarea.h"
#include "diagrampopupmenu.h"
#include "dialogs.h"
#include "diagram_graphic.h"
#include "i18n.h"
#include <gtk--/scrolledwindow.h>
#include <gtk--/viewport.h>
#include <stdio.h>

bool
DiagramWindow::reallyClose()
{
  QuestionDialog *question_dialog;
  bool answer = true;
  string name;

  if (!orderedset->isFileSaved()) {
    name = title; name.erase(0, 1);
    question_dialog =
      new QuestionDialog(name, string(_("Changes were made to")) + " " +
			 name + ".", _("Close anyway?"));
    answer = question_dialog->exec();
    delete question_dialog;
  }
  return answer;
}

gint
DiagramWindow::delete_event_impl(GdkEventAny* e)
{
  if (reallyClose()) return Gtk::Window::delete_event_impl(e);
  return true;
}

DiagramWindow::DiagramWindow(AppWindow *appwin, OrderedSets *oss,
			     OrderedSet *os, int min_width, int min_height)
  : Gtk::Window(GTK_WINDOW_TOPLEVEL)
{
  const int window_frame_width = 2;
  Gtk::ScrolledWindow *scrolledwindow;
  Gtk::Viewport *viewport;
  Coord_t x1, y1, x2, y2, w, h;
  int diagram_border_x, diagram_border_y;

  orderedsets = oss;
  orderedset = os;
  setTitle();
  // scrollview
  scrolledwindow = SigC::manage(new Gtk::ScrolledWindow);
  // automatic scrollbars
  scrolledwindow->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  add(*scrolledwindow);
  scrolledwindow->show();
  viewport = SigC::manage(new Gtk::Viewport
			  (*scrolledwindow->get_hadjustment(),
			   *scrolledwindow->get_vadjustment()));
  // viewport shadow
  viewport->set_shadow_type(GTK_SHADOW_IN);
  scrolledwindow->add(*viewport);
  viewport->show();
  popupmenu = new DiagramPopupMenu(appwin, this, oss, os);
  area =
    SigC::manage(new DiagramArea(this, popupmenu, oss, os, min_width,
				 min_height));
  popupmenu->setArea(area);
  viewport->add(*area);
  orderedset->getDiagram()->getBox(x1, y1, x2, y2);
  w = x2 - x1; h = y2 - y1;
  if (w < min_width) w = min_width;
  if (h < min_height) h = min_height;
  area->getDiagramBorder(diagram_border_x, diagram_border_y);
  set_default_size(w + 2 * diagram_border_x + 2 * window_frame_width, 
		   h + 2 * diagram_border_y + 2 * window_frame_width);
  show();
}

DiagramWindow::~DiagramWindow()
{
  delete popupmenu;
  orderedsets->unrefSet(orderedset);
}

void
DiagramWindow::update()
{
  area->update();
}

void
DiagramWindow::close()
{
  if (reallyClose()) destroy();
}

void
DiagramWindow::setTitle()
{
  char not_saved_mark = '*';
  
  title = orderedset->getFilename();
  if (title == "") title = _("Untitled");
  else title = title.erase(0, title.rfind('/') + 1);
  if (!orderedset->isFileSaved()) {
    if (*(title.begin()) != not_saved_mark) {
      title.insert((string::size_type)0, 1, not_saved_mark);
      set_title(title);
    }
  }
  else set_title(title);
}
