/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// classes of the dialog run ability, message, question, element insert,
// file save as and open dialogs
// abstract class of the file open-save dialog

#ifndef __GTKMM_DIALOGS_H__
#define __GTKMM_DIALOGS_H__

#include <gtk--/dialog.h>
#include <gtk--/button.h>
#include <gtk--/radiobutton.h>
#include <gtk--/entry.h>
#include <gtk--/fileselection.h>
#include <gtk--/main.h>
#include <gtk--/list.h>
#include <gtk--/optionmenu.h>
#include <gtk--/menu.h>
#include <gtk--/notebook.h>
#include <string>
#include "diagram_graphic.h"

using namespace std;

class OrderedSet;
class OrderedSets;
class DiagramArea;
class AppWindow;
class DiagramWindow;

template <class T>
class DialogExecTemplate : public T {
 private:
  bool running;
 protected:
  bool accepted;
  bool modal;
  void done(bool r);
  void accept();
  void reject();
  gint delete_event_impl(GdkEventAny* e) { done(false); return true; }
 public:
  DialogExecTemplate(bool modal_);
  ~DialogExecTemplate();
  bool result() { return accepted; }
  bool exec();
};

template <class T>
void
DialogExecTemplate<T>::done(bool r)
{
  if (!running) return;
  accepted = r;
  if (modal) Gtk::Main::quit();
  hide();
  running = false;
}

template <class T>
void
DialogExecTemplate<T>::accept()
{
  if (!running) return;
  accepted = true;
  if (modal) Gtk::Main::quit();
  hide();
  running = false;
}

template <class T>
void
DialogExecTemplate<T>::reject()
{
  if (!running) return;
  accepted = false;
  if (modal) Gtk::Main::quit();
  hide();
  running = false;
}

template <class T>
DialogExecTemplate<T>::DialogExecTemplate(bool modal_)
{
  // modal
  modal = modal_;
  set_modal(modal);
  accepted = running = false;  
}

template <class T>
DialogExecTemplate<T>::~DialogExecTemplate()
{
  if (!running) return;
  if (modal) Gtk::Main::quit();
}

template <class T>
bool
DialogExecTemplate<T>::exec()
{
  if (running) return accepted;
  show();
  running = true;
  if (modal) Gtk::Main::run();
  return accepted;
}

#define DialogExec DialogExecTemplate<Gtk::Dialog>
#define FileSelectionExec DialogExecTemplate<Gtk::FileSelection>

class MessageDialog : public DialogExec {
 public:
  MessageDialog(string title, string message);
  ~MessageDialog() {}
};

class QuestionDialog : public DialogExec {
 public:
  QuestionDialog(string title, string message, string question);
  ~QuestionDialog() {}
};

class NewElementDialog : public DialogExec {
 protected:
  Gtk::Button *ok_button;
  Gtk::Entry *entry;
  void accept();
  void nameChanged();
 public:
  NewElementDialog(bool rename_dialog = false);
  ~NewElementDialog() {}
  string getElementName();
};

class OpenSaveFileDialog : public FileSelectionExec {
 protected:
  string filetype;
  Gtk::OptionMenu *filetype_optmenu;
  Gtk::Menu *filetype_menu;
  virtual void accept() = 0;
  void fileChanged();
  void filetypeChanged(string type);
 public:
  OpenSaveFileDialog();
  virtual ~OpenSaveFileDialog() {}
  string getFilename();
};

class OpenFileDialog : public OpenSaveFileDialog {
 protected:
  AppWindow *appwindow;
  OrderedSets *orderedsets;
  void accept();
 public:
  OpenFileDialog(AppWindow *appwin, OrderedSets *oss);
  ~OpenFileDialog() {}
  bool openFile(string filename, OrderedSet *&os);
};

class SaveAsFileDialog : public OpenSaveFileDialog {
 protected:
  OrderedSet *orderedset;
  DiagramWindow *diagramwindow;
  void accept();
 public:
  SaveAsFileDialog(OrderedSet *os, DiagramWindow *win, AppWindow *appwin);
  ~SaveAsFileDialog() {}
  bool saveFile(string filename);
};

class NewSetDialog : public DialogExec {
 protected:
  AppWindow *appwindow;
  OrderedSets *orderedsets;
  int width, height;
  Gtk::Button *ok_button;
  Gtk::RadioButton *tree_button, *list_button, *array_button;
  Gtk::Entry *entry_w, *entry_h;
  bool checkValue(string val);
  void accept();
  void sizeChanged();
  void reset();
 public:
  NewSetDialog(AppWindow *appwin, OrderedSets *oss);
  ~NewSetDialog() {}
};

class RearrangeDiagramDialog : public DialogExec {
 protected:
  OrderedSet *orderedset;
  DiagramWindow *diagramwindow;
  Diagram_Graphic::generationParams_t generation_params;
  Gtk::Notebook *notebook;
  Gtk::RadioButton *longest_path_button, *coffman_graham_button,
    *array_button, *list_button, *tree_button, *adjacent_exchange_button,
    *split_button, *barycenter_button, *median_button;
  Gtk::CheckButton *level_min_button, *geom_great_button, *layer_max_button;
  Gtk::Entry *entry;
  void rearrange();
 public:
  RearrangeDiagramDialog(OrderedSet *os, DiagramWindow *win, 
			 AppWindow *appwin);
  ~RearrangeDiagramDialog() {}
};

class SelectPathDialog: public DialogExec {
 protected:
  list<set<Element> > element_paths;
  DiagramArea *diagramarea;
  Gtk::List *paths_list;
  void select();
 public:
  SelectPathDialog(DiagramArea *area);
  ~SelectPathDialog() {}
  void setPaths(list<set<Element> > &paths);
};

class AboutDialog: public DialogExec {
 public:
  AboutDialog();
};

#endif
