/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the generated diagram using the layer method

#ifndef __LAYERDIAGRAM_H__
#define __LAYERDIAGRAM_H__

#include "diagram.h"

class LayerDiagram : public Diagram {
 public:
  enum layering_t {LAYERING_LONGEST_PATH, LAYERING_COFFMAN_GRAHAM};
  typedef unsigned int cg_width_t;
  enum dummy_chains_data_t {DUMMY_D_TREE, DUMMY_D_LIST, DUMMY_D_ARRAY};
  enum crossing_reduction_t {CROSSING_REDUCTION_ADJACENT_EXCHANGE,
			     CROSSING_REDUCTION_SPLIT,
			     CROSSING_REDUCTION_BARYCENTER,
			     CROSSING_REDUCTION_MEDIAN};
  struct generationParams_t : public Diagram::generationParams_t {
    bool from_maximal;
    layering_t layering;
    cg_width_t cg_width;
    dummy_chains_data_t dummy_chains_data_type;
    crossing_reduction_t crossing_reduction;
  };
 protected:
  typedef unsigned int cg_number_t;
  typedef unsigned int crossing_number_t;
  struct cg_NumberElement {
    ElementId element;
    cg_number_t number;
    // comparison function for STL
    bool
    operator <(const cg_NumberElement &e) const
    {
      if (number > e.number) return true;
      if (e.number > number) return false;
      return (element < e.element);
    }
  };
  struct cg_ElemNumberSet {
    ElementId element;
    set<cg_number_t> numbers;
    // comparison function for STL, should be lessNumberSet, but it 
    // would be too complicated ...
    bool
    operator <(const cg_ElemNumberSet &s) const
    {
      if (element < s.element) return true;
      if (s.element < element) return false;
      return (element < s.element);
    }
  };
  OrderData *data;
  set<Vertex_e> verteces;
  OrderData *dummy_chains_data;
  list<list<Vertex_e *> *> layers;
  crossing_number_t **crossing_numbers;
  generationParams_t *generationParams;

  bool isEdge(Vertex_e *vx, Vertex_e *vy);
  // returns: if there is an edge between verteces vx and vy
  void longestPathLayering();
  // does The Longest Path Layering
  // sets up layers
  bool lessNumberSet(set<cg_number_t> s1, set<cg_number_t> s2);
  // returns: if the set s1 is less than the set s2
  // this set order is used by Coffman-Graham algorithm
  void CoffmanGrahamLayering(cg_width_t width);
  // does The Coffman-Graham Layering with at most width elements in
  // each layer
  // sets up layers
  void insertDummyVerteces();
  // inserts dummy verteces to the diagram
  void countCrossingNumbers(list<Vertex_e *> *l1, list<Vertex_e *> *l2);
  // counts crossing numbers for the layer l2
  void adjacentExchangeCrossingReduction(list<Vertex_e *> *layer);
  // does the Adjacent-Exchange sorting crossing reduction method
  void splitCrossingReduction(list<Vertex_e *> *layer);
  // does the Split sorting crossing reduction method
  void barycenterCrossingReduction(list<Vertex_e *> *l1,
				   list<Vertex_e *> *l2);
  // does the Barycenter crossing reduction method
  void medianCrossingReduction(list<Vertex_e *> *l1, list<Vertex_e *> *l2);
  // does the Median crossing reduction method
 public:
  LayerDiagram(OrderData *d);
  ~LayerDiagram() {}
  void generateDiagram(Diagram::generationParams_t *params);
  void getDiagram(set<Vertex_e> &s) { s = verteces; }
  OrderData *getDummyChainsData() { return dummy_chains_data; }
  // returns: dummy chains data (ordered set)
};

#endif
